/*
 * libkysdk-system's Library
 *
 * Copyright (C) 2023-2025, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Yunhe Liu <liuyunhe@kylinos.cn>
 *
 */

body {
  margin: 0;
  padding: 0;
}

.box {
  position: relative;
  height: 800px;
  width: 80%;
  margin: auto;
  margin-top: 50px;
  background: white;
  text-align: center;
}

h1 {
  padding-top: 20px;
  font-family: "Do Hyeon", sans-serif;
}

.spinner {
  border-radius: 50%;
  margin: 20px;
  display: inline-block;
  position: relative;
  height: 60px;
  width: 60px;
  background-color: #e0e0eb;
}

.spinnerOne::after {
  content: "";
  position: absolute;
  border-radius: 50%;
  top: -5px;
  left: -5px;
  height: 60px;
  width: 60px;
  border: 5px solid transparent;
  border-top-color: #7389AE;
  animation: spinneroneani 0.5s linear infinite;
}

@keyframes spinneroneani {
  25% {
    border-top-color: transparent;
    border-right-color: transparent;
    border-bottom-color: transparent;
    border-left-color: #05A8AA;
  }
  50% {
    border-top-color: transparent;
    border-right-color: transparent;
    border-bottom-color: #416788;
    border-left-color: transparent;
    transform: rotate(180deg) scale(1.2);
  }
  75% {
    border-top-color: transparent;
    border-right-color: #2F6690;
    border-bottom-color: transparent;
    border-left-color: transparent;
  }
  100% {
    border-top-color: #7389AE;
    border-right-color: transparent;
    border-bottom-color: transparent;
    border-left-color: transparent;
    transform: rotate(360deg) scale(1);
  }
}
.spinnerTwo:before {
  position: absolute;
  top: 30px;
  left: 27.5px;
  content: "";
  height: 35px;
  width: 5px;
  background-color: #535657;
  transform-origin: 50% 0%;
  border-radius: 8px;
  animation: clockmin 2s linear infinite;
}
.spinnerTwo:after {
  position: absolute;
  content: "";
  height: 35px;
  width: 2px;
  top: 30px;
  left: 29px;
  transform-origin: 50% 0%;
  background-color: #D1462F;
  border-radius: 2px;
  animation: clocksec 0.5s linear infinite;
}
.spinnerTwo span {
  position: absolute;
  top: 10px;
  left: 27.5px;
  height: 20px;
  width: 5px;
  border-radius: 2px;
  background: #535657;
  transform-origin: 50% 100%;
  animation: clockhour 10s linear infinite;
}

@keyframes clockhour {
  0% {
    transform: rotate(0deg);
  }
  100% {
    transform: rotate(360deg);
  }
}
@keyframes clockmin {
  0% {
    transform: rotate(0deg);
  }
  100% {
    transform: rotate(360deg);
  }
}
@keyframes clocksec {
  0% {
    transform: rotate(0deg);
  }
  100% {
    transform: rotate(360deg);
  }
}
.spinnerThree {
  background-color: white;
  border: 3px solid #e0e0eb;
}

.spinnerThree span {
  height: 3px;
  width: 3px;
  border-radius: 50%;
  position: absolute;
  background: #7389AE;
}

.spinnerThree > span:nth-child(1) {
  top: 28px;
  left: 8px;
  animation: bounce 2s linear 0s infinite;
}

.spinnerThree > span:nth-child(2) {
  top: 28px;
  left: 14px;
  animation: bounce 2s linear 0.09s infinite;
}

.spinnerThree > span:nth-child(3) {
  top: 28px;
  left: 20px;
  animation: bounce 2s linear 0.18s infinite;
}

.spinnerThree > span:nth-child(4) {
  top: 28px;
  left: 26px;
  animation: bounce 2s linear 0.27s infinite;
}

.spinnerThree > span:nth-child(5) {
  top: 28px;
  left: 32px;
  animation: bounce 2s linear 0.36s infinite;
}

.spinnerThree > span:nth-child(6) {
  top: 28px;
  left: 38px;
  animation: bounce 2s linear 0.45s infinite;
}

.spinnerThree > span:nth-child(7) {
  top: 28px;
  left: 44px;
  animation: bounce 2s linear 0.54s infinite;
}

.spinnerThree > span:nth-child(8) {
  top: 28px;
  left: 50px;
  animation: bounce 2s linear 0.63s infinite;
}

@keyframes bounce {
  0%, 10%, 100% {
    top: 28px;
    transform: scale(1);
  }
  5% {
    top: 20px;
    transform: scale(2);
  }
}
.spinnerFour {
  overflow: hidden;
  -webkit-mask-image: -webkit-radial-gradient(white, black);
}

.spinnerFour > span {
  height: 60px;
  width: 60px;
  position: absolute;
  border-radius: 50%;
  transform: scale(0);
}

.spinnerFour > span:nth-child(1) {
  top: -2px;
  left: -11px;
  background: #b9dcfa;
  animation: grow 2s linear 0s infinite;
}

.spinnerFour > span:nth-child(2) {
  top: 15px;
  left: 6px;
  background: #4e9e22;
  animation: grow 2s linear 0.5s infinite;
}

.spinnerFour > span:nth-child(3) {
  top: -3px;
  left: -8px;
  background: #1df26c;
  animation: grow 2s linear 1s infinite;
}

.spinnerFour > span:nth-child(4) {
  top: -14px;
  left: -1px;
  background: #683e54;
  animation: grow 2s linear 1.5s infinite;
}

.spinnerFour > span:nth-child(5) {
  top: -3px;
  left: 14px;
  background: #daf610;
  animation: grow 2s linear 2s infinite;
}

.spinnerFour > span:nth-child(6) {
  top: -10px;
  left: 7px;
  background: #cd4902;
  animation: grow 2s linear 2.5s infinite;
}

.spinnerFour > span:nth-child(7) {
  top: 6px;
  left: 7px;
  background: #5d9a5d;
  animation: grow 2s linear 3s infinite;
}

.spinnerFour > span:nth-child(8) {
  top: 9px;
  left: -1px;
  background: #a7bbef;
  animation: grow 2s linear 3.5s infinite;
}

.spinnerFour > span:nth-child(9) {
  top: 3px;
  left: -2px;
  background: #962b6f;
  animation: grow 2s linear 4s infinite;
}

.spinnerFour > span:nth-child(10) {
  top: 2px;
  left: -3px;
  background: #f282be;
  animation: grow 2s linear 4.5s infinite;
}

.spinnerFour > span:nth-child(11) {
  top: 10px;
  left: 7px;
  background: #5dc0f0;
  animation: grow 2s linear 5s infinite;
}

.spinnerFour > span:nth-child(12) {
  top: 1px;
  left: -7px;
  background: #980217;
  animation: grow 2s linear 5.5s infinite;
}

@keyframes grow {
  0% {
    opacity: 1;
    transform: scale(0);
  }
  15% {
    opacity: 1;
  }
  100% {
    transform: scale(1.3);
    opacity: 0;
  }
}
.spinnerFive {
  overflow: hidden;
}

.spinnerFive > span:nth-child(1) {
  position: absolute;
  height: 25px;
  width: 60px;
  background: #7389AE;
  bottom: 0;
  left: 0;
}

.spinnerFive > span:nth-child(2) {
  position: absolute;
  height: 10px;
  width: 10px;
  opacity: 0;
  bottom: 8px;
  left: 33px;
  background: white;
  border-radius: 50%;
  border: 1px solid #7389AE;
  animation: floaty 2s linear 0.8s infinite;
}

.spinnerFive > span:nth-child(3) {
  position: absolute;
  height: 10px;
  width: 10px;
  opacity: 0;
  bottom: 8px;
  left: 22px;
  background: white;
  border-radius: 50%;
  border: 1px solid #7389AE;
  animation: floaty 2s linear 1.6s infinite;
}

.spinnerFive > span:nth-child(4) {
  position: absolute;
  height: 10px;
  width: 10px;
  opacity: 0;
  bottom: 10px;
  left: 33px;
  background: white;
  border-radius: 50%;
  border: 1px solid #7389AE;
  animation: floaty 2s linear 2.4s infinite;
}

.spinnerFive > span:nth-child(5) {
  position: absolute;
  height: 10px;
  width: 10px;
  opacity: 0;
  bottom: 1px;
  left: 32px;
  background: white;
  border-radius: 50%;
  border: 1px solid #7389AE;
  animation: floaty 2s linear 3.2s infinite;
}

.spinnerFive > span:nth-child(6) {
  position: absolute;
  height: 10px;
  width: 10px;
  opacity: 0;
  bottom: 3px;
  left: 36px;
  background: white;
  border-radius: 50%;
  border: 1px solid #7389AE;
  animation: floaty 2s linear 4s infinite;
}

.spinnerFive > span:nth-child(7) {
  position: absolute;
  height: 10px;
  width: 10px;
  opacity: 0;
  bottom: 9px;
  left: 37px;
  background: white;
  border-radius: 50%;
  border: 1px solid #7389AE;
  animation: floaty 2s linear 4.8s infinite;
}

.spinnerFive > span:nth-child(8) {
  position: absolute;
  height: 10px;
  width: 10px;
  opacity: 0;
  bottom: 8px;
  left: 27px;
  background: white;
  border-radius: 50%;
  border: 1px solid #7389AE;
  animation: floaty 2s linear 5.6s infinite;
}

.spinnerFive > span:nth-child(9) {
  position: absolute;
  height: 10px;
  width: 10px;
  opacity: 0;
  bottom: 3px;
  left: 22px;
  background: white;
  border-radius: 50%;
  border: 1px solid #7389AE;
  animation: floaty 2s linear 6.4s infinite;
}

.spinnerFive > span:nth-child(10) {
  position: absolute;
  height: 10px;
  width: 10px;
  opacity: 0;
  bottom: 2px;
  left: 32px;
  background: white;
  border-radius: 50%;
  border: 1px solid #7389AE;
  animation: floaty 2s linear 7.2s infinite;
}

.spinnerFive > span:nth-child(11) {
  position: absolute;
  height: 10px;
  width: 10px;
  opacity: 0;
  bottom: 6px;
  left: 25px;
  background: white;
  border-radius: 50%;
  border: 1px solid #7389AE;
  animation: floaty 2s linear 8s infinite;
}

.spinnerFive > span:nth-child(12) {
  position: absolute;
  height: 10px;
  width: 10px;
  opacity: 0;
  bottom: 6px;
  left: 35px;
  background: white;
  border-radius: 50%;
  border: 1px solid #7389AE;
  animation: floaty 2s linear 8.8s infinite;
}

@keyframes floaty {
  10% {
    opacity: 1;
  }
  25% {
    transform: translate(30%, 0%);
  }
  50% {
    transform: translate(-70%, 0%);
  }
  100% {
    bottom: 60px;
  }
}
